/* this file is part of evince, a gnome document viewer
 *
 *  Copyright (C) 2005 Red Hat, Inc
 *
 * Evince is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Evince is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <glib.h>
#include <glib/gi18n.h>
#include <glib-object.h>

#include "ev-document-types.h"
#include "ev-jobs.h"
#include "ev-job-queue.h"
#include "ev-document-thumbnails.h"
#include "ev-file-chooser.h"

#include <gtk/gtkstock.h>
#include <gtk/gtkfilechooserdialog.h>
#include <libgnomevfs/gnome-vfs-mime-utils.h>
#include <libgnomevfs/gnome-vfs-uri.h>
#include <libgnomevfs/gnome-vfs-utils.h>

#define THUMBNAIL_WIDTH 200

static void ev_file_chooser_init 		   (EvFileChooser *chooser);
static void ev_file_chooser_finalize 		   (GObject *object);
static void ev_file_chooser_class_init		   (EvFileChooserClass *klass);
static void ev_file_chooser_job_completed_callback (EvJobThumbnail      *job,
		    				    gpointer user_data);

struct _EvFileChooser {
    GtkFileChooserDialog parent;

    GtkWidget *image;
    GtkWidget *label;
    GtkWidget *vbox;
    
    EvDocument *document;
    EvJob *job;
};

struct _EvFileChooserClass {
    GtkFileChooserDialogClass parent_class;
};

G_DEFINE_TYPE (EvFileChooser, ev_file_chooser, GTK_TYPE_FILE_CHOOSER_DIALOG)

void
ev_file_chooser_init (EvFileChooser *chooser)
{
	chooser->job = NULL;
	chooser->document = NULL;
        return;
}

void
ev_file_chooser_finalize (GObject *object)
{
	EvFileChooser *chooser = EV_FILE_CHOOSER (object);

	if (chooser->job) {
    	    g_signal_handlers_disconnect_by_func (chooser->job, ev_file_chooser_job_completed_callback,chooser);
    	    ev_job_queue_remove_job (chooser->job);    	
	    chooser->job = NULL;
	}
	
	if (chooser->document) {
	    g_object_unref (chooser->document);
	    chooser->document = NULL;
	}

	
	G_OBJECT_CLASS (ev_file_chooser_parent_class)->finalize (object);
}

static void
ev_file_chooser_class_init (EvFileChooserClass *klass)
{
	GObjectClass *object_class = G_OBJECT_CLASS (klass);

	object_class->finalize = ev_file_chooser_finalize;
}

static void
ev_file_chooser_job_completed_callback (EvJobThumbnail      *job,
	    				gpointer user_data)
{
        EvFileChooser *chooser = EV_FILE_CHOOSER(user_data);

	gtk_image_set_from_pixbuf (GTK_IMAGE (chooser->image), job->thumbnail);
	
	chooser->job = NULL;

	return;
}

static void 
ev_file_chooser_update_preview (EvFileChooser *chooser, gpointer user_data)
{
    char *uri;
    char *mime_type = NULL;
    EvDocument *document = NULL;
    
    if (chooser->job) {
	g_signal_handlers_disconnect_by_func (chooser->job, ev_file_chooser_job_completed_callback,chooser);
	ev_job_queue_remove_job (chooser->job);    	
	chooser->job = NULL;
    }	
    
    if (chooser->document) {
	g_object_unref (chooser->document);
	chooser->document = NULL;
    }
    
    gtk_image_set_from_pixbuf (GTK_IMAGE (chooser->image), NULL);
    gtk_label_set_text (GTK_LABEL (chooser->label), NULL);
	
    uri = gtk_file_chooser_get_preview_uri (GTK_FILE_CHOOSER(chooser));    
    
    if (uri) {
	mime_type = gnome_vfs_get_mime_type (uri);
    }

    if (mime_type != NULL) {
	    const EvinceDocumentType *document_type = evince_document_type_lookup (mime_type);
	    if (document_type) 
		    document = evince_document_type_manufacture_instance (document_type);
    }

    if (document) {
	    
	    chooser->document = document;
    
	    if (ev_document_load (EV_DOCUMENT(document), uri, NULL)) { 
			gchar *label_text;
			gint n_pages;
			
			n_pages = ev_document_get_n_pages (EV_DOCUMENT(document));

			label_text = g_strdup_printf (ngettext("%d page", "%d pages", n_pages), n_pages);	    
    			gtk_label_set_text (GTK_LABEL (chooser->label), label_text);
    		        g_free (label_text);
			
			if (EV_IS_DOCUMENT_THUMBNAILS (document)) {
	    			EvJob *job;
				job = ev_job_thumbnail_new (document, 0, THUMBNAIL_WIDTH);
				chooser->job = job;
				ev_job_queue_add_job (job, EV_JOB_PRIORITY_LOW);
    		    		g_signal_connect (job, "finished",
						  G_CALLBACK (ev_file_chooser_job_completed_callback),
						  chooser);
			} 
	    } else {
			g_object_unref (G_OBJECT(document));
			chooser->document = NULL;
	    }
    } 
    
    g_free (uri);
    g_free (mime_type);
        
    return;    
}

static void
ev_file_chooser_preview_create (EvFileChooser *chooser)
{
    chooser->vbox = gtk_vbox_new (FALSE, 6);
    chooser->label = gtk_label_new (NULL);
    chooser->image = gtk_image_new ();
    
    gtk_box_pack_start (GTK_BOX(chooser->vbox), chooser->label, FALSE, TRUE, 0);
    gtk_box_pack_start (GTK_BOX(chooser->vbox), chooser->image, TRUE, FALSE, 0);
    
    gtk_widget_show_all (chooser->vbox);
    gtk_widget_set_size_request (chooser->vbox, THUMBNAIL_WIDTH + 12, -1);
}

GtkWidget *ev_file_chooser_new (GtkWidget   *toplevel)
{	
	GtkWidget *chooser;

	GtkFileFilter *documents_filter;
	GtkFileFilter *pdf_filter;
	GtkFileFilter *ps_filter;
	GtkFileFilter *pixbuf_filter;
	GtkFileFilter *all_filter;
#ifdef ENABLE_DJVU
	GtkFileFilter *djvu_filter;
#endif
#ifdef ENABLE_DVI
	GtkFileFilter *dvi_filter;
#endif

	chooser = g_object_new (EV_TYPE_FILE_CHOOSER, NULL);

	gtk_window_set_transient_for (GTK_WINDOW (chooser), GTK_WINDOW (toplevel));
	gtk_window_set_title (GTK_WINDOW (chooser), _("Open document"));
	gtk_file_chooser_set_action (GTK_FILE_CHOOSER (chooser), GTK_FILE_CHOOSER_ACTION_OPEN);
	
	gtk_dialog_add_buttons (GTK_DIALOG (chooser), 
					       GTK_STOCK_CANCEL,
					       GTK_RESPONSE_CANCEL,
					       GTK_STOCK_OPEN, GTK_RESPONSE_OK,
					       NULL);

	documents_filter = gtk_file_filter_new ();
	gtk_file_filter_set_name (documents_filter,
				  _("All Documents"));
	gtk_file_filter_add_mime_type (documents_filter, "application/postscript");
	gtk_file_filter_add_mime_type (documents_filter, "application/x-gzpostscript");
	gtk_file_filter_add_mime_type (documents_filter, "image/x-eps");
	gtk_file_filter_add_mime_type (documents_filter, "application/pdf");
#ifdef ENABLE_DVI
	gtk_file_filter_add_mime_type (documents_filter, "application/x-dvi");
#endif
	gtk_file_filter_add_pixbuf_formats (documents_filter);
#ifdef ENABLE_DJVU
	gtk_file_filter_add_mime_type (documents_filter, "image/vnd.djvu");
#endif
	gtk_file_chooser_add_filter (GTK_FILE_CHOOSER (chooser), documents_filter);

	ps_filter = gtk_file_filter_new ();
	gtk_file_filter_set_name (ps_filter, _("PostScript Documents"));
	gtk_file_filter_add_mime_type (ps_filter, "application/postscript");
	gtk_file_filter_add_mime_type (ps_filter, "application/x-gzpostscript");
	gtk_file_filter_add_mime_type (ps_filter, "image/x-eps");
	gtk_file_chooser_add_filter (GTK_FILE_CHOOSER (chooser), ps_filter);

	pdf_filter = gtk_file_filter_new ();
	gtk_file_filter_set_name (pdf_filter, _("PDF Documents"));
	gtk_file_filter_add_mime_type (pdf_filter, "application/pdf");
	gtk_file_chooser_add_filter (GTK_FILE_CHOOSER (chooser), pdf_filter);

#ifdef ENABLE_DVI
	dvi_filter = gtk_file_filter_new ();
	gtk_file_filter_set_name (dvi_filter, _("DVI Documents"));
	gtk_file_filter_add_mime_type (dvi_filter, "application/x-dvi");
	gtk_file_chooser_add_filter (GTK_FILE_CHOOSER (chooser), dvi_filter);
#endif

	pixbuf_filter = gtk_file_filter_new ();
	gtk_file_filter_set_name (pixbuf_filter, _("Images"));
	gtk_file_filter_add_pixbuf_formats (pixbuf_filter);
	gtk_file_chooser_add_filter (GTK_FILE_CHOOSER (chooser), pixbuf_filter);

#ifdef ENABLE_DJVU
	djvu_filter = gtk_file_filter_new ();
	gtk_file_filter_set_name (djvu_filter, _("Djvu Documents"));
	gtk_file_filter_add_mime_type (djvu_filter, "image/vnd.djvu");
	gtk_file_chooser_add_filter (GTK_FILE_CHOOSER (chooser), djvu_filter);
#endif	
	
	all_filter = gtk_file_filter_new ();
	gtk_file_filter_set_name (all_filter, _("All Files"));
	gtk_file_filter_add_pattern (all_filter, "*");
	gtk_file_chooser_add_filter (GTK_FILE_CHOOSER (chooser), all_filter);
	
	gtk_file_chooser_set_filter (GTK_FILE_CHOOSER (chooser), documents_filter);
	
	ev_file_chooser_preview_create (EV_FILE_CHOOSER (chooser));
	gtk_file_chooser_set_preview_widget (GTK_FILE_CHOOSER (chooser), EV_FILE_CHOOSER(chooser)->vbox);	
        gtk_file_chooser_set_use_preview_label (GTK_FILE_CHOOSER (chooser), FALSE);

	g_signal_connect (G_OBJECT (chooser), "update-preview", G_CALLBACK (ev_file_chooser_update_preview), NULL);
	
	return chooser;
}
